/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

class Board
{
  Board(int x, int y)
  {
    this.x = x;
    this.y = y;
    this.s = x * y;
    this.h = new Hex[s];
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        h[i + j * x] = new Hex(i, j);
      }
    }
  }

  // Update flood map
  void update()
  {
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        h[i + j * x].clear();
      }
    }
    // Flood
    ArrayList l = new ArrayList();
    l.add(h[0]);
    while (l.size() > 0)
    {
      Hex h1 = (Hex) l.get(0);
      l.remove(0);
      h1.flood();
      for (int i = 0; i < 6; ++i)
      {
        int ind = (i + h1.r) % 6;
        boolean b = h1.v[ind];
        int tx = h1.x;
        int ty = h1.y;
        if (b)
        {
          switch (i)
          {
            case 0:
              tx += 1;
            break;
            case 1:
              tx += ty % 2;
              ty += 1;
            break;
            case 2:
              tx += (ty % 2) - 1;
              ty += 1;
            break;
            case 3:
              tx -= 1;
            break;
            case 4:
              tx += (ty % 2) - 1;
              ty -= 1;
            break;
            case 5:
              tx += ty % 2;
              ty -= 1;
            break;
          }
          if (tx >= 0 && ty >= 0 && tx < x && ty < y)
          {
            Hex h2 = h[tx + ty * x];
            if (!h2.f && h2.v[(i + 3 + h2.r) % 6])
              l.add(h2);
          }
        }
      }
    }
    // Inverse Flood
    l = new ArrayList();
    l.add(h[x * y - 1]);
    while (l.size() > 0)
    {
      Hex h1 = (Hex) l.get(0);
      l.remove(0);
      h1.iflood();
      for (int i = 0; i < 6; ++i)
      {
        int ind = (i + h1.r) % 6;
        boolean b = h1.v[ind];
        int tx = h1.x;
        int ty = h1.y;
        if (b)
        {
          switch (i)
          {
            case 0:
              tx += 1;
            break;
            case 1:
              tx += ty % 2;
              ty += 1;
            break;
            case 2:
              tx += (ty % 2) - 1;
              ty += 1;
            break;
            case 3:
              tx -= 1;
            break;
            case 4:
              tx += (ty % 2) - 1;
              ty -= 1;
            break;
            case 5:
              tx += ty % 2;
              ty -= 1;
            break;
          }
          if (tx >= 0 && ty >= 0 && tx < x && ty < y)
          {
            Hex h2 = h[tx + ty * x];
            if (!h2.i_f && h2.v[(i + 3 + h2.r) % 6])
              l.add(h2);
          }
        }
      }
    }
  }
  
  float energyIFloodDist()
  {
    float e = 0;
    if (h[0].i_f)
      return e;
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        h[i + j * x].t = false;
      }
    }
    h[x * y - 1].t = true;
    ArrayList l = new ArrayList();
    l.add(h[0]);
    l.add(null);
    while (l.size() > 0)
    {
      Hex h1 = (Hex) l.get(0);
      l.remove(0);
      if (h1 == null)
      {
        e += 1;
        if (l.size() > 0)
          l.add(null);
        continue;
      }
      for (int i = 0; i < 6; ++i)
      {
        int ind = (i + h1.r) % 6;
        int tx = h1.x;
        int ty = h1.y;
        switch (i)
        {
          case 0:
            tx += 1;
          break;
          case 1:
            tx += ty % 2;
            ty += 1;
          break;
          case 2:
            tx += (ty % 2) - 1;
            ty += 1;
          break;
          case 3:
            tx -= 1;
          break;
          case 4:
            tx += (ty % 2) - 1;
            ty -= 1;
          break;
          case 5:
            tx += ty % 2;
            ty -= 1;
          break;
        }
        if (tx >= 0 && ty >= 0 && tx < x && ty < y)
        {
          Hex h2 = h[tx + ty * x];
          if (h2.i_f)
          {
            l.clear();
            break;
          }
          if (h2.t)
            continue;
          h2.t = true;
          l.add(h2);
        }
      }
    }
    return e;
  }

  float energyFloodDist()
  {
    float e = 0;
    if (h[x * y - 1].f)
      return e;
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        h[i + j * x].t = false;
      }
    }
    h[x * y - 1].t = true;
    ArrayList l = new ArrayList();
    l.add(h[x * y - 1]);
    l.add(null);
    while (l.size() > 0)
    {
      Hex h1 = (Hex) l.get(0);
      l.remove(0);
      if (h1 == null)
      {
        e += 1;
        if (l.size() > 0)
          l.add(null);
        continue;
      }
      for (int i = 0; i < 6; ++i)
      {
        int ind = (i + h1.r) % 6;
        int tx = h1.x;
        int ty = h1.y;
        switch (i)
        {
          case 0:
            tx += 1;
          break;
          case 1:
            tx += ty % 2;
            ty += 1;
          break;
          case 2:
            tx += (ty % 2) - 1;
            ty += 1;
          break;
          case 3:
            tx -= 1;
          break;
          case 4:
            tx += (ty % 2) - 1;
            ty -= 1;
          break;
          case 5:
            tx += ty % 2;
            ty -= 1;
          break;
        }
        if (tx >= 0 && ty >= 0 && tx < x && ty < y)
        {
          Hex h2 = h[tx + ty * x];
          if (h2.f)
          {
            l.clear();
            break;
          }
          if (h2.t)
            continue;
          h2.t = true;
          l.add(h2);
        }
      }
    }
    return e;
  }
  
  float computeEnergy()
  {
    e = energySurface() + energyFloodDist() + energyIFloodDist();
    return e;
  }

  float energySurface()
  {
    float e = 0;
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        if (h[i + j * x].f || h[i + j * x].i_f)
          e += 1;
      }
    }
    e /= x * y;
    if (!h[x * y - 1].f)
      e = 2 - e;
    return e;
  }

  void draw()
  {
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        pushMatrix();
        translate(x_offset + i * hex_x + (hex_x / 2) * (j % 2), y_offset + j * (hex_s + hex_h));
        h[i + j * x].draw();
        popMatrix();
      }
    }
  }
  
  void reset()
  {
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        h[i + j * x].reset();
      }
    }    
  }
  
  int x;
  int y;
  int s;
  float e; // energy;
  Hex h[];
}

