/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

class GASolver extends Solver
{
  GASolver(Board b)
  {
    super(b);
    bt = new Board(x, y);
    p = new ArrayList();
    e = new ArrayList();

    // Initialize bt with same hexes as b
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        for (int n = 0; n < 6; n++)
        {
          bt.h[i + j * x].v[n] = b.h[i + j * x].v[n];
        }
      }
    }
    reset();
  }
  
  int[] extractIndividual(Board b)
  {
    int[] ind = new int[x * y];
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        ind[i + j * x] = b.h[i+ j * x].r;
      }
    }
    return ind;    
  }

  int[] randomIndividual()
  {
    int[] ind = new int[x * y];
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        ind[i + j * x] = int(random(6));
      }
    }
    return ind;
  }
  
  void apply(Board b, int[] ind)
  {
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        b.h[i + j * x].r = ind[i + j * x];
        b.h[i + j * x].invalidate();
      }
    }    
  }

  void reset()
  {
    super.reset();
    p.clear();
    e.clear();
    g = 1;
    int n = 0;
    int[] ind = extractIndividual(b);
    apply(bt, ind);
    bt.update();
    float en = bt.computeEnergy();
    while (n < s_ga_elites)
    {
      p.add(ind);
      e.add(en);
      n++;
    }
    while (n < s_ga_size)
    {
      ind = randomIndividual();
      apply(bt, ind);
      bt.update();
      en = bt.computeEnergy();
      // Sorted insert
      int i = 0;
      while (i < e.size() && (Float) e.get(i) < en)
        i++;
      p.add(i, ind);
      e.add(i, en);
      n++;
    }
  }

  int[] combine(int[] i1, int[] i2)
  {
    int[] ind = new int[x * y];
    for (int i = 0; i < x; i++)
    {
      for (int j = 0; j < y; j++)
      {
        int p = i + j * x;
        int r = round(random(1));
        ind[p] = i1[p] * r + i2[p] * (1 - r);
        if (random(1) < s_ga_pm)
          ind[p] += int(random(6));
        ind[p] = (ind[p] % 6);
      }
    }
    return ind;
  }

  void step()
  {
    ArrayList np = new ArrayList();
    ArrayList ne = new ArrayList();
    boolean found = false;
    // Generate children
    int n = 0;
    while (n < s_ga_elites)
    {
      // Sorted insert
      np.add(p.get(n));
      ne.add(e.get(n));
      n++;
    }
    while (n < s_ga_size)
    {
      int p1 = int(random(s_ga_parent_pool));
      int p2 = int(random(s_ga_parent_pool));
      int[] ind = combine((int[]) p.get(p1), (int[]) p.get(p2));
      apply(bt, ind);
      bt.update();
      float en = bt.computeEnergy();
      // Sorted insert
      int i = 0;
      while (i < ne.size() && (Float) ne.get(i) <= en)
        i++;
      np.add(i, ind);
      ne.add(i, en);
      n++;
    }
    g += 1;
    if (np.get(0) != p.get(0))
    {
      apply(b, (int[]) np.get(0));
      super.e = (Float) ne.get(0);
    }
    p = np;
    e = ne;
  }

  void draw()
  {
    fill(0);
    textFont(arial, 32);  
    text("E = " + (int(b.e * 100) / 100.0), 10, 40);
    text("G = " + g, 200, 40); 
  }
  
  int g;
  Board bt;
  ArrayList p; // Solutions
  ArrayList e; // Energy levels
}
