/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

class Hex
{
  Hex(int x, int y)
  {
    this.x = x;
    this.y = y;
    v = new boolean[6];
    r = 0;
    f = false;
    df = false;
    i_f = false;
    i_df = false;
    over = false;
    valid = false;
    randomize();
  }

  void randomize()
  {
    for (int i = 0; i < 6; ++i)
      v[i] = random(1) < 0.5;
    invalidate();
  }

  void flood()
  {
    f = true;
  }

  void iflood()
  {
    i_f = true;
  }

  void clear()
  {
    f = false;
    i_f = false;
  }

  void turnLeft()
  {
    r = (r + 1) % 6;
    invalidate();
  }
  
  void rotate(int tr)
  {
    r = (r + tr + 6) % 6;  
    invalidate();  
  }
  
  void turnRight()
  {
    r = (r + 5) % 6;
    invalidate();
  }

  void reset()
  {
    r = 0;
    invalidate();
  }
  
  void invalidate()
  {
    valid = false;
  }
  
  boolean isOver()
  {
    float dx = (mouseX - (x_offset + x * hex_x + (hex_x / 2) * (y % 2)));
    float dy = (mouseY - (y_offset + y * (hex_s + hex_h)));
    float l = hex_x / 2;
    boolean b = (dx * dx + dy * dy < l * l);
    if (over != b)
      invalidate();
    over = b;
    return b;
  }

  void draw()
  {
    boolean over = isOver();
    if (f != df || i_f != i_df)
      invalidate();
    df = f;
    i_df = i_f;

    if (valid)
      return;
    if (f)
    {
      fill(50, 50, 180);
      if (over)
        fill(80, 80, 210);
      if (i_f)
      {
        fill(50, 180, 50);
        if (over)
          fill(80, 210, 80);
      }
    }
    else if (i_f)
    {
      fill(180, 50, 50);
      if (over)
        fill(210, 80, 80);
    }
    else
    {
      fill(160);
      if (over)
        fill(200);
    }
    stroke(0);
    strokeWeight(hex_border_weight);
    beginShape();
    vertex(- hex_x / 2, - hex_s / 2);
    vertex(0, - hex_y / 2);
    vertex(hex_x / 2, - hex_s / 2);
    vertex(hex_x / 2, hex_s / 2);
    vertex(0, hex_y / 2);
    vertex(- hex_x / 2, hex_s / 2);
    vertex(- hex_x / 2, - hex_s / 2);
    endShape();
    fill(0);
    ellipse(0, 0, hex_center_size, hex_center_size);
    strokeWeight(hex_pipe_weight);
    for (int i = 0; i < 6; ++i)
    {
      int ind = (i + r) % 6;
      boolean b = v[ind];
      if (b)
      {
        line(0, 0, hex_x / 2 * cos(i * PI / 3), hex_x / 2 * sin(i * PI / 3));
      }
    }
    valid = true;
  }

  int x;
  int y;
  int r; // Rotation
  boolean v[]; // Accessible edges
  boolean f; // Flooded
  boolean df; // Displayed flooded?
  boolean i_f; // Flooded (inverse)
  boolean i_df; // Displayed inverse flooded?
  boolean t; // Temporary
  boolean over;
  boolean valid;
}

