/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

// Note: Using background images is as expensive as drawing.

Board b;
Solver s;
boolean solve = false;
PFont arial = createFont("Arial", 32);  

void setup()
{
  size(800, 600);
  frameRate(30);
  smooth();
  background(127, 127, 127);
  strokeCap(SQUARE);
  b = new Board(sx, sy);
  s = new SASolver(b);
}

void draw()
{
  //background(127, 127, 127);
  b.update();
  b.computeEnergy();
  if (solve)
    s.step();
  b.draw();
  
  if (!click_handled && mousePressed)
  {
    click_handled = true;
  }
  if (click_handled && !mousePressed)
  {
    click_handled = false;
  }
  
  // Set the font and its size (in units of pixels)
  noStroke();
  fill(127, 127, 127);
  rect(0, 0, w, 40);
  fill(0);
  textFont(arial, 32);  
  text("FPS: " + int(frameRate), w - 150, 40);
  if (solve)
    s.draw();
  else
    text("E = " + (int(b.e * 100) / 100.0), 10, 40);
}

void mouseClicked()
{
  // Not very optimized
  for (int i = 0; i < b.x; i++)
  {
    for (int j = 0; j < b.y; j++)
    {
      Hex hx = b.h[i + j * b.x];
      if (!hx.isOver())
        continue;
      // Handle clicks
      if (mouseButton == RIGHT)
        hx.turnLeft();
      else if (mouseButton == LEFT)
        hx.turnRight();
    }
  }
}

void keyPressed() 
{
  if (key == 'r')
  {
    b = new Board(sx, sy);
    s = new Solver(b);
    solve = false;
  }
  if (key == 's')
  {
    s = new SASolver(b);
    solve = true;
  }
  if (key == 'g')
  {
    s = new GASolver(b);
    solve = true;
  }
  if (key == 'p')
  {
    solve = !solve;
  }
  if (key == 'h')
  {
    s.heatUp();
  }
  if (key == 'c')
  {
    s.coolDown();
  }
  if (key == 't')
  {
    b.reset();
    s.reset();
    solve = false;
  }
}

