/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

class SASolver extends Solver
{
  SASolver(Board b)
  {
    super(b);
  }
  
  boolean[] check(Board b, int i, int j)
  {
    Hex h1 = b.h[i + j * x];
    boolean[] bt = new boolean[2];
    bt[0] = false;
    bt[1] = false;
    for (int n = 0; n < 6; ++n)
    {
      int tx = h1.x;
      int ty = h1.y;
      switch (n)
      {
        case 0:
          tx += 1;
        break;
        case 1:
          tx += ty % 2;
          ty += 1;
        break;
        case 2:
          tx += (ty % 2) - 1;
          ty += 1;
        break;
        case 3:
          tx -= 1;
        break;
        case 4:
          tx += (ty % 2) - 1;
          ty -= 1;
        break;
        case 5:
          tx += ty % 2;
          ty -= 1;
        break;
      }
      if (tx >= 0 && ty >= 0 && tx < b.x && ty < b.y)
      {
        Hex h2 = b.h[tx + ty * x];
        if (h2.f)
          bt[0] = true;
        if (h2.i_f)
          bt[1] = true;
      }
    }
    return bt;
  }
  
  void step()
  {
    int n = x * y * s_tries; // Max tries
    float te = e;
    while (n > 0)
    {
      int i = int(random(x));
      int j = int(random(y));
      int r = int(random(6));
      Hex h = b.h[i + j * x];
      boolean bt[] = check(b, i, j);
      if (h.f || h.i_f || bt[0] || bt[1])
      {
        h.rotate(r);
        b.update();
        te = b.computeEnergy();
        if (te < e || random(1) < exp((e - te) / t))
          break;
        else
        h.rotate(-r);
        b.update();
      }
      n -= 1;
    }
    e = te;
    t *= s_cf;
  }
}
