/*
  Processing.js Pipe Game Demo
  Copyright Florent D'Halluin 2010

  This file is part of the Processing.js Pipe Game Demo.
  See README.txt for help.
  
  COPYRIGHT NOTICE
------------------------------------------------------------------------  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------
 */

class Solver
{
  // Basic simulated annealing
  Solver(Board b)
  {
    x = b.x;
    y = b.y;
    this.b = b;
    t = s_t; 
    e = b.e;
  }
  
  void reset()
  {
    t = s_t;
    e = b.e;
  }
  
  void step()
  {
    int n = x * y * s_tries; // Max tries
    float te = e;
    while (n > 0)
    {
      int i = int(random(x));
      int j = int(random(y));
      int d = int(random(2));
      Hex h = b.h[i + j * x];
      if (d == 0)
        h.turnLeft();
      else
        h.turnRight();
      b.update();
      te = b.computeEnergy();
      if (te < e || random(1) < exp((e - te) / t))
        break;
      else
      {
        if (d == 0)
          h.turnRight();
        else
          h.turnLeft();
      }
      n -= 1;
    }
    e = te;
    t *= s_cf;
  }
  
  void heatUp()
  {
    t = min(s_t, t * s_mhf);
  }
  
  void coolDown()
  {
    t = min(s_t, t * s_mcf);
  }
  
  void draw()
  {
    fill(0);
    textFont(arial, 32);  
    text("E = " + (int(e * 100) / 100.0), 10, 40);
    text("T = " + (int(t * 10000) / 10000.0), 200, 40); 
  }
  
  int x;
  int y;
  float t; // Temperature
  float e; // Energy
  Board b;
}
